# Kinetica 3D Volumetric Demo

This project demonstrates the process of generating 3D geometry from data in Kinetica. In this tutorial we will populate a Kinetica database with a minimal data set (< 1 million points) and generate a 3D model representing a **hollow tile** visualization.

Before starting you will need the following:

* Access to a Kinetica instance. If you don’t have this you can you can [install one locally](https://www.kinetica.com/docs/install/docker.html) from a docker container.  
* A copy of the tutorial code. (Location TBD)
* [Java 1.8 SDK](https://www.oracle.com/technetwork/java/javase/downloads/index.html) or later.

The image below is an example of the visualization.

![3dTile](3dtile.jpg)

## Setup
Extract the distribution zip to a local directory. You should see the following files.

```
[~/gvis-demo-1.0]$ ls -l
total 24088
-rw-r--r--   1 chadjuliano  staff   148K Oct 26 22:28 3dtile.jpg
-rw-r--r--   1 chadjuliano  staff   5.3K Oct 25 15:20 README.md
-rw-r--r--   1 chadjuliano  staff   283B Oct 25 10:36 demo.yml
-rw-r--r--   1 chadjuliano  staff    12M Oct 30 15:22 gvis-demo-1.0.jar 
```

Edit the `demo.yml` file with the API connection parameters for your Kinetica instance.

```
# Configuration for GVIS Demo

# put Kientica connection details here
kineticaConnection:
    url: http://localhost:9191
    user: admin
    password: admin
```

## Data Organization

Within a basin the geological formations are organized into layers which is important to scientists because the minerals of interest are often hiding in pockets between the layers. Each layer represents a horizontal plane having different elevations.  
When data for a visualization is selected the horizontal boundaries are specified in longitude and latitude and the depth is specified with a starting and ending layer. 

The image below shows a sequence of layers with elevations rendered in 3D. 
The visualizations in this tutorial use data that is organized in a 3 dimensional grid. Each point or voxel in the grid has an integer based (x, y, layer) position and a corresponding fractional decimal (latitude, longitude, depth) position. 

![Layer stack](layer_stack.png)

## Generating Simulated Data

The demo data loader will create a table `demo_basin_lod0` and generate simulated volumetric data using [OpenSimplexNoise](https://en.wikipedia.org/wiki/OpenSimplex_noise). It will generate points with varying elevations and an attribute column that will be used to indicate a measurement of the basin at the point.

The table will be populated with 684,478 rows. This is enough data for the bounding region of a volume with dimensions 493x246x299. In a real use case this volume would contain 36 million points but to reduce memory requirements it will only populate the exterior of the volume. In the visualization step we will request a volume that is aligned with this boundary resulting to populate the hollow tile.

The table below shows the important columns of `demo_basin_lod0`:

| **Column** | **Type** | **Description** | 
| --- | --- | --- |
| x | float | Geographical latitude (WGS84) | 
| y | float | Geographical longitude (WGS84) | 
| z | float | Depth in feet | 
| scan_x | int | x grid position (NAD27) | 
| scan_y | int | y grid position (NAD27) | 
| layer | int | z grid position | 
| porosity | float | Measured attribute | 

It is important to note that each row has integer based (x, y, z) position within the grid and a corresponding geographic coordinate given as (scan_x, scan_y, depth). The integer grid must be evenly spaced and it is used by the visualizer to retrieve orthogonal planes. When a plane, such as a single layer, is retrieved from the table it will represent a surface with different elevations.

The loader will procedurally generate terrain based on a random seed number. In the below example we pass 999 as the random seed but you can change this to any number. Each seed results in an entirely different data set. To start the loader run the command below.

```
[~/gvis-demo-1.0]$ java -cp ./gvis-demo-1.0.jar com.kinetica.gvis.demo.BasinLoader 999
INFO  c.k.g.d.ConfigCommon - Got config file: /Users/chadjuliano/Local/workspace/apc-3dvis/gvis-demo/target/gvis-demo-1.0/demo.yml
INFO  o.h.v.i.util.Version - HV000001: Hibernate Validator 1.0
INFO  c.k.g.d.ConfigCommon - Connecting to Kinetica: admin@http://localhost:9191
INFO  c.k.g.d.BasinLoader - Clearing table <demo_basin_lod0>
INFO  c.k.g.d.BasinLoader - Creating table <demo_basin_lod0> with type ID <9355741195739408978>
INFO  c.k.g.d.BasinLoader - Populating layer for: layer=299 area=x[897500,946700] y[514100,538600]
(...)
INFO  c.k.g.r.AttributeMetadata - Attrib JSON: {"porosity":{"stdv":0.11274789048865107,"mean":-0.01722200461847618,"skew":0.16019293759632575,"estimated_cardinality":0.0,"sum":585377.1266559015,"variance":0.012712086809640855,"max":0.0,"min":0.0,"kurtosis":2.5751307358448483,"count":684478.0,"colormap":"HSB"}}
INFO  c.k.g.d.BasinLoader - Loading done!
INFO  c.k.g.d.BasinLoader - Job completed.
```

## Generating The Model

The visualizer will connect to the Kinetica database and fetch orthogonal X-Y, X-Z, and Y-Z planes of rows from the table. It will then generate textures and 3D geometry to produce a 3D tile. The demo visualizer is a simple wrapper for the `gvis-modeler` library which handles the data retrieval and 3D geometry setup. The following table lists some important components.

| **Name** | **Description** | 
| --- | --- |
| gvis-modeler | Top level library that retrieves basin and well data from Kinetica and renders the visualization | 
| gvis-qbuilder | Used by the modeler to build Kinetica REST queries and retrieve data | 
| [jgltf-mesh](https://github.com/chadj2/jgltf-mesh/) | Used by the modeler to for low level geometry calculations | 
| [jgltf](https://github.com/javagl/JglTF) | Serializes the data to the glTF file format | 

The following code shows an example of how the `gvis-modeler` is invoked. It is passed a rectangle indicating the boundaries of the XY-plane (in NAD27 coordinates). The generated tile will include all layers of the Z axis.

```
GridRect _rect = new GridRect(897500, 514100, 196800, 98200);
LOG.info("Generating tile for rectangle: {}", _rect.toString());

this._modeler.centerAbout(_rect);
this._modeler.addTile(_rect);

final File _outFile = GltfTileGenerator.getFile("demo_tile");
this._modeler.writeGltf(_outFile);
```

To run the model generator execute the following command:

```
[~/gvis-demo-1.0]$ java -cp ./gvis-demo-1.0.jar com.kinetica.gvis.demo.GltfTileGenerator
INFO  c.k.g.d.ConfigCommon - Got config file: /Users/chadjuliano/Local/workspace/apc-3dvis/gvis-demo/target/gvis-demo-1.0/demo.yml
INFO  o.h.v.i.util.Version - HV000001: Hibernate Validator 1.0
INFO  c.k.g.d.ConfigCommon - Connecting to Kinetica: admin@http://localhost:9191
INFO  c.k.g.p.GridRasterizer - Headless mode: true
INFO  c.k.g.p.PlaneResourcesBase - setSubSample: (xy=1 layer=1) => (xy=100, layer=1, lod=0)
(...)
INFO  c.k.mesh.GltfWriter - Wrote glTF: /Users/chadjuliano/Local/workspace/apc-3dvis/gvis-demo/target/gvis-demo-1.0/./out_gltf/demo_tile.gltf
INFO  c.k.g.d.GltfTileGenerator - Job completed.
```

The program should create a file `./out_gltf/demo_tile.gltf` containing the 3D model.

```
[~/gvis-demo-1.0]$ ls -l out_gltf/
total 2816
-rw-r--r--  1 chadjuliano  staff   1.4M Oct 30 17:43 demo_tile.gltf
```

## References

* [glTF Overview](https://www.khronos.org/gltf/)
* [glTF MeshBuilder](https://github.com/chadj2/jgltf-mesh/)
* [Oil and Gas Engineering](https://www.oilandgaseng.com/articles/active-analytics-can-drive-better-business-decisions/)


